#############################################################
# Canvases for displaying Pythia physics sub-processes
# Author: J. Couchman
#############################################################
from Tkinter import *
import string
#############################################################
# class: BlankCan
# Inherits from Canvas, sets canvas size and colour
# Author: J. Couchman
#############################################################
class BlankCan(Canvas):
    def __init__(self,parent):
        Canvas.__init__(self,parent)
        self.config(width=250,height=25,bg='#46d900')
#############################################################
# class: MainProCan
# Inherits from BlankCan,
# Takes argument of process number, places number in correct
# on canvas
# Author: J. Couchman
#############################################################
class MainProCan(BlankCan):
    def __init__(self,parent,no):
        BlankCan.__init__(self,parent)
        self.create_text(40,12,text=no,fill='black',font=('times',15,'bold'))
#############################################################
# class: TitleProCan
# Inherits from BlankCan,
# Takes string argument title, the physics process group 
# Author: J. Couchman
#############################################################
class TitleProCan(BlankCan):
    def __init__(self,parent,title):
        BlankCan.__init__(self,parent)
        self.create_text(125,12,text=title,fill='red',font=('times',15,'bold'))
#############################################################
# class: PhysicsTextCan
# Inherits from MainProCan,
# Takes string argument m_text, the physics process if
# written in words. will convert p| into pt pymbol
# Author: J. Couchman
#############################################################
class PhysicsTextCan(MainProCan):
    def __init__(self,parent,no,m_text):
        MainProCan.__init__(self,parent,no)
        i = 62
        for t in m_text:
            if t != '|':
                self.create_text(i,12,text=t,fill='black',font=('times',15,'bold'))
            else:
                self.create_text(i,17,text=chr(94),fill='black',font=('symbol',15,'bold'))
            i += 7
#############################################################
# class: PhysicsProCan
# Inherits from MainProCan,
# Takes tuple of strings argument protext. This places
# members of tuple on canvas in order.
# Author: J. Couchman
# Special Symbols(have to be in following order)
# ----------------------------------------------
# % followed by number, will offset the tuple members
#   X-position by ammount specified
# { turns font into italic
# _ subscript
# ^ superscript
# [ go to symbol font (greek etc.)
# arrow make an arrow
# /pm make a plus-minus
#############################################################            
class PhysicsProCan(MainProCan):
    def __init__(self,parent,no,protext):
        MainProCan.__init__(self,parent,no)
        xpos = 55
        for char in protext:
#SET DEFAULT VALUES-------------------------------------
            txtsiz = 15
            ypos = 12
            fonttyp = 'times'
            bold = 'bold'
            mov =10
#DO SPECIAL FORMATING-----------------------------------
            if char[:1] == '%':
                char = char[1:]
                if char[:1] == '-':
                    char = char[1:]
                    xpos -= int(char[:1])
                else:
                    xpos += int(char[:1])
                char = char[1:]
            if char[:1] == '{':
                bold = 'bold italic'
                char = char[1:]
            if char[:1] == '_':
                try:   
                    if char[-1:] in string.uppercase:
                        txtsiz = 11
                    else:
                        ttxsiz = 12
                except:
                    txtsiz = 12
                ypos = 16
                mov = 6
                char = char[1:]
            if char[:1] == '^':
                txtsiz = 11
                ypos = 6
                mov = 7
                char = char[1:]
            if char[:1] == '[':
                fonttyp = 'symbol'
                char = char[1:]

            if char == 'arrow':
                fonttyp = 'symbol'
                char = chr(174)
                mov = 15
            if char == '/pm':
                fonttyp = 'symbol'
                char = chr(177)
            if char == '/mp':
                fonttyp = 'symbol'
                char = chr(177)
 
            try:   
                if char in string.uppercase:
                    xpos += 2
            except:
                    pass
#MAKE TEXT-----------------------------------------------
            xpos += mov
            if char[-3:] == 'bar':
                self.create_text(xpos,3,text='-',fill='black',font=m_font)
                char = char[:-3]
            m_text = char
            m_font = (fonttyp,txtsiz,bold)
            self.create_text(xpos,ypos,text=m_text,fill='black',font=m_font)
#DO SPECIAL END STUFF FORMATTING-------------------------
            if char == chr(174):
                xpos += 5
            else:
                try:
                    if char in string.uppercase:
                        xpos += 1
                except:
                    pass
if __name__=='__main__':
    root = Tk()
    m_main = Frame(root)
    m_main.pack()

    PhysicsProCan(m_main,12,('a','_i','b','_j','=','x','^2')).pack(side=TOP)
    PhysicsProCan(m_main,13,('a','_i','b','_j','=','Y','^2')).pack(side=TOP)
    PhysicsProCan(m_main,99,('a','_q','b')).pack(side=TOP)
    PhysicsProCan(m_main,1,('X','_[g','=','y','+','[k', '+', 'W','_q')).pack(side=TOP)
    PhysicsProCan(m_main,123,('q','_i','q','_j','arrow','[g','[g')).pack(side=TOP)
    PhysicsProCan(m_main,123,('W','H','arrow','W','b','bbar')).pack(side=TOP)
    PhysicsProCan(m_main,123,('W','H','arrow','W','C','Bbar')).pack(side=TOP)
    PhysicsTextCan(m_main,123,('single diffraction (AX)')).pack(side=TOP)
    
    root.mainloop()
